package models

import "gogs.mikescher.com/BlackForestBytes/goext/rfctime"

type CrawlerLogType string //@enum:type

const (
	CrawlerLogTypeHTTPCall CrawlerLogType = "HTTP_CALL"
	CrawlerLogTypeMessage  CrawlerLogType = "MESSAGE"
	CrawlerLogTypeError    CrawlerLogType = "ERROR"
)

type CrawlerLog struct {
	CrawlerLogID       CrawlerLogID            `json:"id"                  bson:"_id,omitempty"`
	CreationTime       rfctime.RFC3339NanoTime `json:"creationTime"        bson:"creationTime"`
	CrawlerID          CrawlerID               `json:"crawlerID"           bson:"crawlerID"`
	CrawlerExecutionID CrawlerExecutionID      `json:"crawlerExecutionID"  bson:"crawlerExecutionID"`
	Type               CrawlerLogType          `json:"type"                bson:"type"`
	DataHTTPCall       *CrawlerLogHTTPCall     `json:"dataHTTPCall"        bson:"dataHTTPCall"`
	DataMessage        *CrawlerLogMessage      `json:"dataMessage"         bson:"dataMessage"`
	DataError          *CrawlerLogError        `json:"dataError"           bson:"dataError"`
}

type CrawlerLogHTTPCall struct {
	Success       bool                 `json:"success"           bson:"success"`
	ErrorStr      *string              `json:"errorStr"          bson:"errorStr"`
	CallingMethod string               `json:"callingMethod"     bson:"callingMethod"`
	Duration      rfctime.SecondsF64   `json:"duration"          bson:"duration"`
	Request       CrawlerHTTPRequest   `json:"request"           bson:"request"`
	Response      *CrawlerHTTPResponse `json:"response"          bson:"response"`
}

type CrawlerHTTPRequest struct {
	Method      string                 `json:"method"            bson:"method"`
	URL         string                 `json:"url"               bson:"url"`
	RequestBody string                 `json:"requestBody"       bson:"requestBody"`
	Header      []CrawlerHTTPHeaderDef `json:"header"            bson:"header"`
}

type CrawlerHTTPResponse struct {
	ResponseStatus int                    `json:"responseStatus"     bson:"responseStatus"`
	ResponseBody   *string                `json:"responseBody"       bson:"responseBody"`
	Header         []CrawlerHTTPHeaderDef `json:"header"             bson:"header"`
	Message        string                 `json:"message"            bson:"message"`
}

type CrawlerHTTPHeaderDef struct {
	Key    string
	Values []string
}

type CrawlerLogMessage struct {
	Identifier string `json:"identifier" bson:"identifier"`
	Message    string `json:"message"    bson:"message"`
	Extra      any    `json:"extra"      bson:"extra"`
}

type CrawlerLogError struct {
	Message string `json:"message"    bson:"message"`
	Error   string `json:"error"      bson:"error"`
	Trace   string `json:"trace"      bson:"trace"`
	Extra   any    `json:"extra"      bson:"extra"`
}
